{{
*******************************************
* Prototype_Accel_DataLogger.spin  Ver 1  *
* Author: Michael Lefebvre                *
* Copyright (c) 2010 Grame                *
*******************************************

   History:
        Version 1 - (04-09-2010) initial concept         
        
}}

CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

'--------IO Pins-----------
  'MMA7455
  CLKPIN        = 0
  DATAPIN       = 1
  CSPIN         = 2
 
  'Memory Stick Datalogger
  rxpin       = 10              'Receive pin on the Propeller
  txpin       = 13              'Transmit pin on the Propeller
  ctspin      = 12

  'Led
  ready_led       = 19
  initialise_led  = 18
  recording_led   = 17

  'Button
  start_stop_button = 20      

  
  LF            = 10            ' line feed
  CR            = 13            ' carriage return
  
  X_OFFSETVAL   = 26            ' X-Axis offset compensation value
  Y_OFFSETVAL   = 52            ' Y-Axis offset compensation value
  Z_OFFSETVAL   = -26           ' Z-Axis offset compensation value
{{
        The Offset value is an iterative process.
        To adjust the Offset, set X_OFFSETVAL, Y_OFFSETVAL, and Z_OFFSETVAL to Zero
        and run the program.

        Multiply the returned value by 2 and subtract from the corresponding current
        offset value.

                  0 - -28 = 28

        Note: - The Offset is valid only at Zero-g, so you will need to orient the
                Z axis so that it is perpendicular to the influence of gravity.

              - At rest, it is possible to see a g force greater than 1 or less than
                1 (where 1g returns 63) because depending on where you are, the force
                of gravity can be stronger or weaker depending on your elevation.   
  
}}
VAR

  long  XYZData[3]

  long  filenameaddr

  long stack[10]

  long cog
  
OBJ

  Spi           : "MMA7455L_SPI_v2"      'Used to Communicate to the Accelerometer
  Usb           : "Autodatalogger"
  Debug         : "FullDuplexSerialPlus"

  'vp           : "Conduit"

PUB main | Time

  'vp.share(@XYZData[0], @XYZData[2])

  dira[start_stop_button]~

  LedInit

  waitcnt(clkfreq + cnt)

  outa[initialise_led]~~


  filenameaddr := string("acqui.txt")
  
  'Initialize the devices

  Debug.start(31, 30, 0, 57600)
  waitcnt(clkfreq*6 + cnt)

  Debug.str(string("Accelerometer DataLogger"))
  Debug.tx(13)

  waitcnt(clkfreq*2 + cnt)
  Debug.str(string("Initialisation..."))
  Debug.tx(13)

  USB.init(rxpin, txpin, ctspin, filenameaddr, 10_000)

  SPI.start(CLKPIN, DATAPIN, CSPIN)

  waitcnt(clkfreq/100+cnt)

  'Write the X, Y, and Z offset compensation values
  SPI.write(SPI#XOFFL, X_OFFSETVAL)                         
  SPI.write(SPI#XOFFH, X_OFFSETVAL >> 8)
  SPI.write(SPI#YOFFL, Y_OFFSETVAL)
  SPI.write(SPI#YOFFH, Y_OFFSETVAL >> 8)
  SPI.write(SPI#ZOFFL, Z_OFFSETVAL)
  SPI.write(SPI#ZOFFH, Z_OFFSETVAL >> 8)

  outa[initialise_led]~
  outa[ready_led]~~
  
  repeat
    Debug.str(string("Ready..."))
    Debug.tx(13)
    
    repeat

      if ina[start_stop_button] == 1
        outa[ready_led]~
        waitcnt(clkfreq + cnt)
        quit
      
      waitcnt(clkfreq/10 + cnt)


    cog := cognew(StartRead10BitData, @stack[0]) + 1

    'Set up the number of columns to log

    USB.addfield(@XYZData[0], 4, string("Accel_X"))
    USB.addfield(@XYZData[1], 4, string("Accel_Y"))
    USB.addfield(@XYZData[2], 4, string("Accel_Z"))
    

    USB.start

    debug.str(string("Logging data now"))
    debug.tx(13)

    Time := cnt
    
    repeat
      if ina[start_stop_button] == 1
        debug.str(string("Stopping..."))
        debug.tx(13)
        outa[recording_led]~
        outa[ready_led]~~
        waitcnt(clkfreq + cnt)

        USB.stop
        debug.str(string("Stop USB done..."))
        debug.tx(13)
        'StopRead10BitData
        debug.str(string("Stop Accelerometer done..."))
        debug.tx(13)

        quit
                        
      debug.tx(".")
      !outa[recording_led]
      waitcnt(Time += clkfreq/2)

    debug.str(string("Acquisition Completed"))
    debug.tx(13)


PRI StartRead10BitData
  repeat
    SPI.write(SPI#MCTL, (%0110 << 4)|SPI#G_MODE)                 'Initialize the Mode Control register
    DataIn_High      := SPI.read(SPI#XOUTH)                       'repeat for X-axis
    DataIn_Low       := SPI.read(SPI#XOUTL)
    XYZData[0] := DataIn
    XYZData[0] := ~XYZData[0] + 511                               'Convert from -511 to 511 data to 0 to 1023 (Always positive)
    DataIn_High      := SPI.read(SPI#YOUTH)                       'repeat for Y-axis
    DataIn_Low       := SPI.read(SPI#YOUTL)
    XYZData[1] := DataIn
    XYZData[1] := ~XYZData[1] + 511                               'Convert from -511 to 511 data to 0 to 1023 (Always positive)
    DataIn_High      := SPI.read(SPI#ZOUTH)                       'and Z-axis
    DataIn_Low       := SPI.read(SPI#ZOUTL)
    XYZData[2] := DataIn
    XYZData[2] := ~XYZData[2] + 511                               'Convert from -511 to 511 data to 0 to 1023 (Always positive)
    waitcnt(clkfreq/100+cnt)

PRI StopRead10BitData
  if cog
    cogstop(cog~ - 1)

PRI LedInit
  dira[17..19]~~                

  outa[19]~~                'LEDs flash
  waitcnt(clkfreq/10 + cnt)
  !outa[18..19]
  waitcnt(clkfreq/10 + cnt)
  !outa[17..18]
  waitcnt(clkfreq/10 + cnt)
  !outa[17..18]
  waitcnt(clkfreq/10 + cnt)
  !outa[18..19]
  waitcnt(clkfreq/10 + cnt)
  !outa[18..19]
  waitcnt(clkfreq/10 + cnt)
  !outa[17..18]
  waitcnt(clkfreq/10 + cnt)
  !outa[17..18]
  waitcnt(clkfreq/10 + cnt)
  !outa[18..19]
  waitcnt(clkfreq/10 + cnt)
  outa[19]~
  

DAT

DataIn        word              ' Data positioning trick to convert Words to Bytes
DataIn_Low    byte    0         ' or Bytes to a Word without using any math overhead
DataIn_High   byte    0